/*
 * Network model with automatic layout
 *
 * $Header: /usr/src/mash/repository/vint/nam-1/anetmodel.h,v 1.3 1997/09/27 23:03:33 haoboy Exp $
 */

#ifndef nam_anetmodel_h
#define nam_anetmodel_h

#include "netmodel.h"

class AutoNetModel : public NetModel {
public:
	AutoNetModel();
	virtual ~AutoNetModel();

	static int RANDOM_SEED_;
	static int AUTO_ITERATIONS_;
	static int INCR_ITERATIONS_;
	static double INIT_TEMP_; // follow Elan's 
	static double MINX_; 
	static double MAXX_;
	static double MINY_;
	static double MAXY_;
	static double min(double a, double b) {
		return (a < b) ? a : b;
	}
	static double max(double a, double b) {
		return (a > b) ? a : b;
	}

	void handle(const TraceEvent& e, double now, int direction);

protected:
	void layout();
	void relayout();
	virtual void embed();
	void scale_estimate();
	virtual void placeEdge(Edge* e, Node* src) const;
	virtual void placeAgent(Agent *a, Node *src) const;
	void placeAllAgents(Node *src) const;
	void placeEverything();

	// attractive and repulsive forces
	double ATT (double d, double k) { return d * d / k; }
	double REP (double d, double k) { return k * k / d; }
	void cool();
	void initEmbedding();
	
	// we need our own 'cmd layout'
	int command(int argc, const char*const* argv); 

private:
	int iterations_;
	int nNodes_;
	double optk_;    // k used in computing att/rep forces
	double optka_;
	double optkr_;
	double temp_;
	double kc_;
	double kca_;
	double kcr_;
};


#endif // nam_anetmodel_h
